/*
                Sun Public License Notice

The contents of this file are subject to the Sun Public License
Version 1.0 (the "License"). You may not use this file except in
compliance with the License. A copy of the License is available at
http://www.sun.com/

The Original Code is NetBeans. The Initial Developer of the Original
Code is Sun Microsystems, Inc. Portions Copyright 1997-2004 Sun
Microsystems, Inc. All Rights Reserved.
*/
package org.netbeans.modules.snipe;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.openide.util.Utilities;
import org.openide.windows.IOProvider;
import org.openide.windows.OutputWriter;

/**
 * Implementation of a Snipe Editor panel.
 *
 * @author Rich Unger
 */
public class SnipeEditor { 
    private SnipeDataObject m_snipe;

    public SnipeEditor() { 
    } 

    public SnipeDataObject getSnipe() { 
        return m_snipe;
    } 

    public void setSnipe(SnipeDataObject dobj) { 
        m_snipe = dobj;
    } 

    public void open() { 
        log("Opening Snipe Editor");
    } 

    public boolean close() { 
        log("Closing Snipe Editor");
        return true;
    } 

    public JComponent getPanel() { 
        return new SnipePanel();
    } 

    private void log(String str) { 
        OutputWriter out = IOProvider.getDefault().getStdOut();
        out.println(str);
        out.flush();
    } 

    private class SnipePanel extends JPanel { 
        private Image m_image;
        private int m_imagex, m_imagey;

        public SnipePanel() { 
            super();
            m_image = Utilities.loadImage("org/netbeans/modules/snipe/resources/snipe.gif");
            m_imagex = m_image.getWidth(this);
            m_imagey = m_image.getHeight(this);
            addMouseListener(new MouseAdapter() { 
                public void mouseClicked(MouseEvent evt) { 
                    Point p = getImageLocation();
                    int x = evt.getX();
                    int y = evt.getY();

                    if( p.x <= x && x <= (p.x + m_imagex) &&
                        p.y <= y && y <= (p.y + m_imagey) ) { 
                        log("You shot the snipe!");
                    } 
                }
            }); 
        } 

        public void paint(Graphics g) { 
            Point p = getImageLocation();
            g.drawImage(m_image, p.x, p.y, Color.white, this);
        } 

        private Point getImageLocation() { 
            int panelx = getWidth();
            int panely = getHeight();
            int x = Math.max(0, (panelx - m_imagex) / 2);
            int y = Math.max(0, (panely - m_imagey) / 2);
            return new Point(x, y);
        } 
    } 
} 
